/**************************************************************************
*  Copyright (c) 2015-2016 by Michael Fischer (www.emb4fun.de)
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  15.11.2015  mifi  First Version for the Altera DE1 Board.
*  12.03.2016  mifi  Tested with TinyCTS v0.11.
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdint.h>
#include "tal.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

#define LED_LD0_ON()    { IOWR_ALTERA_AVALON_PIO_SET_BITS(PIO_LED_GREEN_BASE, 0x01);   }
#define LED_LD0_OFF()   { IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(PIO_LED_GREEN_BASE, 0x01); } 

#define LED_LD1_ON()    { IOWR_ALTERA_AVALON_PIO_SET_BITS(PIO_LED_GREEN_BASE, 0x02);   }
#define LED_LD1_OFF()   { IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(PIO_LED_GREEN_BASE, 0x02); } 

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/* 
 * Some TASK variables like stack and task control block.
 */
static OS_STACK (LEDStack,   TASK_LED_STK_SIZE);
static OS_STACK (StartStack, TASK_START_STK_SIZE);

static os_tcb_t TCBLed;
static os_tcb_t TCBStartTask;


/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  SysTickHandler                                                       */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void SysTickHandler (void* context)
{
   /* Clear the interrupt */
   IOWR_ALTERA_AVALON_TIMER_STATUS(SYSTICK_BASE, 0);
   
   /* 
    * Dummy read to ensure IRQ is negated before the ISR returns.
    * The control register is read because reading the status
    * register has side-effects per the register map documentation.
    */
   IORD_ALTERA_AVALON_TIMER_CONTROL(SYSTICK_BASE);

   /* Cal timer callback */   
   OSTimerCallback(); 
    
} /* SysTickHandler */

/*************************************************************************/
/*  InitSysTick                                                          */
/*                                                                       */
/*  Init the SysTick timer.                                              */
/*                                                                       */                                                                       
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void InitSysTick (void)
{
   alt_irq_context   status;
   static uint8_t   bInitDone = 0;
   uint32_t         dHirResPeriod;
   uint16_t         wValue;
   
   if (0 == bInitDone)
   {
      bInitDone = 1;

      /* Calculate timer period value */
      dHirResPeriod = (SYSTICK_FREQ / OS_TICKS_PER_SECOND) - 1;
      
      /* Set period register */
      IOWR_ALTERA_AVALON_TIMER_PERIODH(SYSTICK_BASE, 0);      
      IOWR_ALTERA_AVALON_TIMER_PERIODL(SYSTICK_BASE, dHirResPeriod);

      /* Set control register, stop timer */      
      wValue = ALTERA_AVALON_TIMER_CONTROL_STOP_MSK |
               ALTERA_AVALON_TIMER_CONTROL_CONT_MSK |
               ALTERA_AVALON_TIMER_CONTROL_ITO_MSK;
      IOWR_ALTERA_AVALON_TIMER_CONTROL(SYSTICK_BASE, wValue);
      
      /* Clear status */
      IOWR_ALTERA_AVALON_TIMER_STATUS(SYSTICK_BASE, 0);
  
      /* Set SysTick interrupt handler */
      status = alt_irq_disable_all();  
      alt_ic_isr_register(SYSTICK_IRQ_INTERRUPT_CONTROLLER_ID, SYSTICK_IRQ, SysTickHandler, 0x0, 0x0);
      alt_ic_irq_disable(SYSTICK_IRQ_INTERRUPT_CONTROLLER_ID, SYSTICK_IRQ);
      alt_irq_enable_all(status);

      /* Set control register, start timer */      
      wValue = ALTERA_AVALON_TIMER_CONTROL_START_MSK |
               ALTERA_AVALON_TIMER_CONTROL_CONT_MSK  |
               ALTERA_AVALON_TIMER_CONTROL_ITO_MSK;
      IOWR_ALTERA_AVALON_TIMER_CONTROL(SYSTICK_BASE, wValue);

      /* Enable SysTick interrupt */   
      alt_ic_irq_enable(SYSTICK_IRQ_INTERRUPT_CONTROLLER_ID, SYSTICK_IRQ);
   }      
} /* InitSysTick */

/***************************************************************************/
/*  LEDInit                                                                */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void LEDInit (void)
{
   /* Switch off 7-segment display */
   IOWR_ALTERA_AVALON_PIO_DATA(PIO_7SEG_BASE, 0xFFFFFFFF);
   
   /* Switch of all green LEDs */   
   IOWR_ALTERA_AVALON_PIO_DATA(PIO_LED_GREEN_BASE, 0x0000);

   /* Switch of all red LEDs */   
   IOWR_ALTERA_AVALON_PIO_DATA(PIO_LED_RED_BASE, 0x0000);

} /* LEDInit */

/*************************************************************************/
/*  HardwareInit                                                         */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void HardwareInit (void)
{
   /* Init LED */
   LEDInit();

   /* Init SysTick */
   InitSysTick();

} /* HardwareInit */

/*************************************************************************/
/*  LEDTask                                                              */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static void LEDTask (void *p)
{
   (void)p;
   
   while(1)
   {
      LED_LD0_ON();
      OSTimeDly(TASK_LED_DELAY_MS);
      LED_LD0_OFF();
      OSTimeDly(TASK_LED_DELAY_MS);
   }
} /* LEDTask */

/*************************************************************************/
/*  StartTask                                                            */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static void StartTask (void *p)
{
   (void)p;
   
   /*
    * The StartTask will be used to start all other tasks in the system.
    * At the end the priority will be set to "IDLE" priority.
    */
    
   /*
    * Start the System ticker
    */
   OSSysTickStart();    

   /* Create the LED task */
   OSTaskCreate(&TCBLed, LEDTask, NULL, TASK_LED_PRIORITY,
                LEDStack, sizeof(LEDStack), 
                "LEDTask");

   OSTaskChangePriority(TASK_START_PRIORITY_IDLE);

   while (1)
   {
      LED_LD1_ON();
      OSTimeDly(TASK_START_DELAY_MS);
      LED_LD1_OFF();
      OSTimeDly(TASK_START_DELAY_MS);
   }
} /* StartTask */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  main                                                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
int main (void)
{
   /*
    * Before the application which is using the TinyCTS can be started, 
    * the hardware of the target must be initialized. The CPU is still
    * initialized by the Nios II SBT startup. But the timer tick
    * must be init too. Even the GPIO can be init here.
    */
   HardwareInit();

   /* Init the TinyCTS */   
   OSInit();
   
   /* 
    * Create the StartTask.
    * The StartTask is the one and only task which 
    * will start all other tasks of the system.
    */
   OSTaskCreate(&TCBStartTask, StartTask, NULL, TASK_START_PRIORITY,
                StartStack, sizeof(StartStack), 
                "StartTask");

   /* 
    * OSStart must be the last function here.
    *  
    * Fasten your seatbelt, engine will be started...
    */        
   OSStart();
   
   /*
    * This return here make no sense.
    * But to prevent the compiler warning:
    *    "return type of 'main' is not 'int'
    * We use an int as return :-)
    */ 
   return(0); /*lint !e527*/
} /* main */

/*** EOF ***/
